import requests

from sota_extractor.errors import HttpClientError
from sota_extractor.scrapers.utils import date_from_timestamp
from sota_extractor.taskdb.v01 import SotaRow, Dataset, Task, Link, TaskDB


URL = "https://sheng-z.github.io/ReCoRD-explorer/"
JSON_URL = (
    "https://raw.githubusercontent.com/sheng-z/ReCoRD-explorer/master/"
    "codalab/output/out-v1.0.json"
)

TASK_NAME = "Common Sense Reasoning"
DATASET_NAME = "ReCoRD"


def get_sota_rows(data):
    rows = data["leaderboard"]

    sota_rows = []
    for row in rows:
        # HACK: This hack with the timezone is needed because the person who
        #       maintains the leaderboard uses local timezone. He just runs the
        #       gulp html generation script in (from github profile) Redmond/WA
        #       which is in the US/Pacific zone so we need to parse the
        #       timestamp like we are in that zone to get the same dates they
        #       get.
        date = date_from_timestamp(
            row.get("submission", {}).get("created", None), tz="US/Pacific"
        )

        description = row.get("submission", {}).get("description", "").strip()
        # This peace of a the code is taken from gulpfile.js translated to py
        model_name = description[: description.rfind("(")].strip()
        # _first_part = description[description.rfind("(") + 1 :]
        # _institution = _first_part[: _first_part.rfind(")")]
        if description.rfind("http") != -1:
            link = description[description.rfind("http") :].strip()
        else:
            link = ""

        sota_rows.append(
            SotaRow(
                model_name=model_name,
                paper_title=link,
                paper_url=link,
                paper_date=date,
                metrics={
                    "EM": str(row.get("scores", {}).get("exact_match", 0)),
                    "F1": str(row.get("scores", {}).get("f1", 0)),
                },
            )
        )
    return sota_rows


def record() -> TaskDB:
    """Extract ReCoRD SOTA tables."""
    try:
        data = requests.get(JSON_URL).json()
    except Exception as e:
        raise HttpClientError(message=str(e))

    dataset = Dataset(name=DATASET_NAME, is_subdataset=False,)
    task = Task(name=TASK_NAME)
    task.datasets = [dataset]
    task.source_link = Link(title="ReCoRD Leaderboard", url=URL)

    # scrape the evaluation values on the two datasets
    dataset.sota.metrics = ["EM", "F1"]

    dataset.sota.rows = get_sota_rows(data)

    tdb = TaskDB()
    tdb.add_task(task)
    return tdb



import re

import requests

from sota_extractor.errors import HttpClientError
from sota_extractor.scrapers.utils import date_from_timestamp
from sota_extractor.taskdb.v01 import SotaRow, Dataset, Task, Link, TaskDB


URL = "https://hotpotqa.github.io/"
JSON_URL = (
    "https://raw.githubusercontent.com/hotpotqa/hotpotqa.github.io/master/"
    "leaderboard-utils/fullwiki-leaderboard.json"
)

TASK_NAME = "Question Answering"
DATASET_NAME = "HotpotQA"


def get_sota_rows(data):
    rows = data["leaderboard"]

    sota_rows = []
    for row in rows:
        # FIXME: The generation of the page uses local timezone and there are a
        #        couple of them that match the current state but I have no idea
        #        which is the correct one.
        date = date_from_timestamp(
            row.get("submission", {}).get("created", None)
        )

        description = row.get("submission", {}).get("description", "").strip()
        # This peace of a the code is taken from gulpfile.js translated to py
        model_name = description[: description.find("(")].strip()
        match = re.search(r"\[(.*?)\]\s*\((.*?)\)", description)
        if match is not None:
            paper, link = match.groups()
            paper = paper.strip().strip("()").strip()
            link = link.strip()
        else:
            paper = ""
            link = ""

        sota_rows.append(
            SotaRow(
                model_name=model_name,
                paper_title=paper,
                paper_url=link,
                paper_date=date,
                metrics={
                    "ANS-EM": str(row.get("scores", {}).get("ans_em", 0)),
                    "ANS-F1": str(row.get("scores", {}).get("ans_f1", 0)),
                    "SUP-EM": str(row.get("scores", {}).get("sup_em", 0)),
                    "SUP-F1": str(row.get("scores", {}).get("sup_f1", 0)),
                    "JOINT-EM": str(row.get("scores", {}).get("joint_em", 0)),
                    "JOINT-F1": str(row.get("scores", {}).get("joint_f1", 0)),
                },
            )
        )
    return sota_rows


def hotpotqa() -> TaskDB:
    """Extract HotpotQA SOTA tables."""
    try:
        data = requests.get(JSON_URL).json()
    except Exception as e:
        raise HttpClientError(message=str(e))

    dataset = Dataset(name=DATASET_NAME, is_subdataset=False,)
    task = Task(name=TASK_NAME)
    task.datasets = [dataset]
    task.source_link = Link(title="HotpotQA Leaderboard", url=URL)

    # scrape the evaluation values on the two datasets
    dataset.sota.metrics = [
        "ANS-EM",
        "ANS-F1",
        "SUP-EM",
        "SUP-F1",
        "JOINT-EM",
        "JOINT-F1",
    ]

    dataset.sota.rows = get_sota_rows(data)

    tdb = TaskDB()
    tdb.add_task(task)
    return tdb



import click
from sota_extractor import scrapers
from sota_extractor.consts import Format
from sota_extractor import serialization
from sota_extractor.commands.cli import cli
from sota_extractor.errors import catch_errors


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/eff.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def eff(output, fmt):
    """Extract EFF SOTA tables."""
    serialization.dump(tdb=scrapers.eff(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/redditsota.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def reddit(output, fmt):
    """Extract Reddit SOTA tables."""
    serialization.dump(tdb=scrapers.reddit(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/snli.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def snli(output, fmt):
    """Extract SNLI SOTA tables."""
    serialization.dump(tdb=scrapers.snli(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/squad.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def squad(output, fmt):
    """Extract SQUAD SOTA tables."""
    serialization.dump(tdb=scrapers.squad(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/cityscapes.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def cityscapes(output, fmt):
    """Extract Cityscapes SOTA tables."""
    serialization.dump(tdb=scrapers.cityscapes(), output=output, fmt=fmt)


@cli.command("nlp-progress")
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/nlp-progress.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def nlp_progress(output, fmt):
    """Extract NLP Progress SOTA tables."""
    serialization.dump(tdb=scrapers.nlp_progress(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/smcalflow.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def smcalflow(output, fmt):
    """Extract Smcalflow SOTA tables."""
    serialization.dump(tdb=scrapers.smcalflow(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/record.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def record(output, fmt):
    """Extract Record SOTA tables."""
    serialization.dump(tdb=scrapers.record(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/hotpotqa.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def hotpotqa(output, fmt):
    """Extract hotpotqa SOTA tables."""
    serialization.dump(tdb=scrapers.hotpotqa(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/coqa.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def coqa(output, fmt):
    """Extract coqa SOTA tables."""
    serialization.dump(tdb=scrapers.coqa(), output=output, fmt=fmt)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/chexpert.json",
    help="Output filename.",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(Format),
    default=Format.json,
    help="Output format.",
)
@catch_errors
def chexpert(output, fmt):
    """Extract chexpert SOTA tables."""
    serialization.dump(tdb=scrapers.chexpert(), output=output, fmt=fmt)



import re
import logging
from xml.etree.ElementTree import ElementTree
from typing import Optional, List, Union, Tuple

from sota_extractor.taskdb.v01 import Sota, SotaRow, Dataset, Link


logger = logging.getLogger(__name__)


def nlp_progress_link() -> Link:
    return Link(
        title="NLP-progress",
        url="https://github.com/sebastianruder/NLP-progress",
    )


class Model:
    AUTHORS_AND_OR_YEARS = re.compile(
        r"""
        \(                        # Open brace
            \s*                   # Optional space
            (?P<author>[^\)]*?)?  # Non greedy optional author
            \s*                   # Optional space
            (?P<year>\d{4})?      # Optional year
            \s*                   # Optional space
        \)                        # Close brace
        """,
        re.VERBOSE | re.UNICODE,
    )

    def __init__(
        self,
        name: Optional[str] = None,
        author: Optional[str] = None,
        uses_additional_data: bool = False,
    ):
        self.name = name
        self.author = author
        self.uses_additional_data = uses_additional_data

    @staticmethod
    def _ws(s: str) -> str:
        """Normalize whitespace."""
        return re.sub(r"\s+", " ", s).strip()

    @classmethod
    def parse(cls, s: str) -> Optional["Model"]:
        s = cls._ws(s)
        if "with additional unlabeled data" in s.lower():
            uses_additional_data = True
            s = cls._ws(
                re.sub(
                    "with additional unlabeled data", "", s, 1, re.IGNORECASE
                )
            )
        else:
            uses_additional_data = False

        years = []
        authors = []
        for match in cls.AUTHORS_AND_OR_YEARS.finditer(s):
            d = match.groupdict()
            if d["author"] is not None and d["author"].strip() != "":
                authors.append(d["author"].replace("et al.", "").strip())
            if d["year"] is not None:
                years.append(d["year"])
        s = cls._ws(cls.AUTHORS_AND_OR_YEARS.sub("", s))

        name = s
        for year in years:
            name += f" ({year})"

        author = ", ".join(authors)
        return cls(
            name=name, author=author, uses_additional_data=uses_additional_data
        )


class Text:
    def __init__(self, text: str = "", links: List[Link] = None):
        self.text = text
        self.links = links or []

    @classmethod
    def _unwind(cls, el: ElementTree, keep_links) -> "Text":
        start = el.text or ""
        if el.tag == "a":
            links = [Link(title=el.text, url=el.attrib.get("href", None))]
        else:
            links = []

        children = ""
        for child in el:
            t = cls._unwind(child, keep_links=keep_links)
            children += t.text
            links.extend(t.links)

        end = el.tail or ""

        if el.tag == "a" and keep_links:
            return cls(
                text=f"[{start}]({el.attrib.get('href', '')}){children}{end}",
                links=links,
            )
        return cls(text=start + children + end, links=links)

    @classmethod
    def parse(
        cls, elements: Union[ElementTree, List[ElementTree]], keep_links=True
    ) -> "Text":
        if isinstance(elements, ElementTree):
            elements = [elements]
        text = cls()
        for el in elements:
            t = cls._unwind(el, keep_links=keep_links)
            text.text += t.text
            text.links.extend(t.links)
        return text


def parse_sota(table: ElementTree) -> Sota:
    headers = [
        (header.text or "").strip() for header in table.findall("thead/tr/th")
    ]
    headers_sanitized = [
        re.sub(r"\s+", "", header.lower()) for header in headers
    ]

    if "model" in headers_sanitized:
        model_idx = headers_sanitized.index("model")
    else:
        logger.error(
            "Model name not found in this SOTA table, skipping. Headers: %s",
            headers_sanitized,
        )
        return Sota()

    if "paper/source" in headers_sanitized:
        paper_idx = headers_sanitized.index("paper/source")
    elif "paper" in headers_sanitized:
        paper_idx = headers_sanitized.index("paper")
    else:
        logger.error(
            "Paper not found in this SOTA table, skipping. Headers: %s",
            headers_sanitized,
        )
        return Sota()

    if "code" in headers_sanitized:
        code_idx = headers_sanitized.index("code")
    else:
        code_idx = None

    metrics_idx = sorted(
        set(range(len(headers))) - {model_idx, paper_idx, code_idx}
    )

    metrics_names = [headers[i] for i in metrics_idx]

    sota = Sota(metrics=metrics_names)

    for row in table.findall("tbody/tr"):
        cols = list(row)
        if len(cols) != len(headers):
            # Skip row not enough columns.
            continue

        paper = Text.parse(cols[paper_idx], keep_links=False)
        model = Model.parse(cols[model_idx].text)
        if model is None:
            # Skip row model cannot be determined.
            continue

        sota.rows.append(
            SotaRow(
                model_name=model.name,
                paper_title=paper.text,
                paper_url=paper.links[0].url if len(paper.links) > 0 else "",
                metrics={
                    name: (cols[metrics_idx[i]].text or "").strip()
                    for i, name in enumerate(metrics_names)
                },
            )
        )
    return sota


def parse_subdatasets(
    parent: Dataset, pairs: List[Tuple[ElementTree, ElementTree]]
) -> List[Dataset]:
    subdatasets = []
    for p, table in pairs:
        strong = p.find("strong")
        if strong is None:
            continue
        subdatasets.append(
            Dataset(
                name=strong.text.strip().strip(":"),
                is_subdataset=True,
                parent=parent,
                sota=parse_sota(table),
            )
        )
    return subdatasets



import os
import glob
import logging
import tempfile
import subprocess

from sota_extractor.errors import DataError
from sota_extractor.taskdb.v01 import TaskDB
from sota_extractor.consts import NLP_PROGRESS_REPO
from sota_extractor.scrapers.nlp_progress.markdown import parse_file

logger = logging.getLogger(__name__)


def nlp_progress() -> TaskDB:
    """Parse a the whole nlp progress repo or a single markdown file.

    Checkouts the nlp progress git repository and parses all the markdown files
    in it.

    Returns:
        TaskDB: Populated task database.
    """
    tdb = TaskDB()

    with tempfile.TemporaryDirectory() as tmpdir:
        repo_path = os.path.join(tmpdir, "nlp-progress")
        cp = subprocess.run(
            ["git", "clone", NLP_PROGRESS_REPO, repo_path], capture_output=True
        )
        if cp.returncode != 0:
            logger.error("stdout: %s", cp.stdout)
            logger.error("stderr: %s", cp.stderr)
            raise DataError("Could not clone the NLP Progress repository.")

        filenames = glob.glob(os.path.join(repo_path, "english", "*.md"))

        for filename in filenames:
            file_tdb = parse_file(filename)
            for task in file_tdb.tasks.values():
                tdb.add_task(task)

    return tdb



from typing import Optional, List

from sota_extractor.taskdb.v01 import Task, Dataset


def fix_dataset(dataset: Dataset) -> Optional[Dataset]:
    """Walk through the dataset and return a valid one.

    Valid dataset contains only subdatasets with non empty sota tables or,
    if it has no subdatasets, its sota table has to be non-empty.
    """
    dataset.subdatasets = [
        subdataset
        for subdataset in dataset.subdatasets
        if len(subdataset.sota.rows) > 0
    ]

    if len(dataset.subdatasets) > 0:
        return dataset

    if len(dataset.sota.rows) > 0:
        return dataset

    return None


def fix_datasets(datasets: List[Dataset]) -> List[Dataset]:
    """Filter out invalid datasets."""
    return [
        dataset
        for dataset in [fix_dataset(dataset) for dataset in datasets]
        if dataset is not None
    ]


def fix_subtasks(subtasks: List[Task]) -> List[Task]:
    """Filter out invalid subtasks."""
    valid_subtasks = []

    for subtask in subtasks:
        subtask.datasets = fix_datasets(subtask.datasets)
        if len(subtask.datasets) > 0:
            valid_subtasks.append(subtask)

    return valid_subtasks


pop_subtasks = {
    "Language Modeling",
    "Domain Adaptation",
}

subtask_to_task = {
    "Question Answering": ["Reading Comprehension"],
}


def fix_task(task: Task) -> List[Task]:
    """Return a valid task.

    To be a valid task it has to have a dataset with sota tables or a sub-task
    with dataset with a valid table.
    """
    task.datasets = fix_datasets(task.datasets)
    task.subtasks = fix_subtasks(task.subtasks)

    # Hierarchy change
    if task.name in pop_subtasks:
        return task.subtasks

    tasks: List[Task] = []

    # Hierarchy change for some
    if task.name in subtask_to_task:
        tasks = [
            subtask
            for subtask in task.subtasks
            if subtask.name in subtask_to_task[task.name]
        ]
        task.subtasks = [
            subtask
            for subtask in task.subtasks
            if subtask.name not in subtask_to_task[task.name]
        ]

    if len(task.datasets) > 0 or len(task.subtasks) > 0:
        tasks.append(task)

    return tasks
